package com.me.asteroids;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.AsteroidComponent;
import com.me.asteroids.components.BulletComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.components.model.PolygonModel;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;

import static com.me.asteroids.Constants.rand;

public class EntityFactory {

    private static final Vector2 tmp = new Vector2();

    private static Entity createEntity(Engine engine) {
        Entity entity = engine.createEntity();
        if (Constants.DEBUG) {
            System.out.println("Spawned entity: " + entity);
        }
        return entity;
    }

    public static Entity createPlayer(Engine engine) {
        PositionComponent position = new PositionComponent();
        position.position = new Vector2(Constants.HALF_WIDTH, Constants.HALF_HEIGHT);
        position.rotation = 90;

        VelocityComponent velocity = new VelocityComponent();
        velocity.velocity = new Vector2(0f, 0f);
        velocity.maxVelocity = 400f;

        ModelComponent model = new ModelComponent();
        model.model = new PolygonModel(Color.WHITE);
        model.model.setVertices(new float[] {
                0f, 4f,         // tip
                -2.5f, -4f,     // bottom left
                -1f, -2.5f,     // indent
                1f, -2.5f,      // indent
                2.5f, -4f,      // bottom right
        });
        model.model.setScale(5);

        AccelerationComponent accel = new AccelerationComponent();
        accel.acceleration = new Vector2(0,1f);

        Entity player = createEntity(engine);
        player.addComponent(position);
        player.addComponent(velocity);
        player.addComponent(model);
        player.addComponent(accel);
        player.addComponent(new PlayerComponent());
        return player;
    }

    public static Entity createBullet(Engine engine, Entity player) {
        float[] modelVertices = player.getComponent(ModelComponent.class).model.getVertices();
        float rotation = player.getComponent(PositionComponent.class).rotation;

        Vector2 direction = Utils.setUnitVectorAngle(tmp, rotation);

        VelocityComponent velocity = new VelocityComponent();
        velocity.velocity = new Vector2(direction).scl(500);

        PositionComponent position = new PositionComponent();
        position.position = new Vector2(modelVertices[0], modelVertices[1]).add(direction.scl(4));
        position.rotation = rotation;

        ModelComponent model = new ModelComponent();
        model.model = new PolygonModel(Color.YELLOW);
        model.model.setVertices(new float[] {
                1f, 0f,
                -1f, 0f,
                -1f, -4f,
                1f, -4f,
        });
        model.model.setRotation(position.rotation);
        model.model.setPosition(position.position);


        Entity bullet = createEntity(engine);
        bullet.addComponent(position);
        bullet.addComponent(velocity);
        bullet.addComponent(model);
        bullet.addComponent(new BulletComponent());
        return bullet;
    }

    public static Entity createAsteroid(Engine engine) {
        // Creates an asteroid entity with position and velocity unset - the AsteroidSpawningSystem
        // is responsible for setting its position and velocity
        PositionComponent position = new PositionComponent();
        VelocityComponent velocity = new VelocityComponent();

        ModelComponent model = new ModelComponent();
        model.model = new PolygonModel(Color.WHITE);
        int size = rand.nextInt(30, 60);
        model.model.setVertices(new AsteroidFactory()
                .setVertexCount(rand.nextInt(16, 24))
                .setSize(size)
                .setSizeVariation(size * 0.7f)
                .sizeRelativeToLast()
                .generate());

        Entity asteroid = createEntity(engine);
        asteroid.addComponent(position);
        asteroid.addComponent(velocity);
        asteroid.addComponent(model);
        asteroid.addComponent(new AsteroidComponent());
        return asteroid;
    }

}
